import pyglet
from pyleap.shape.shape import Shape

__all__ = ['Point']


class Point(Shape):
    """ 点 Point """

    def __init__(self, x=100, y=100, color="orange", point_size=1):
        super().__init__(color, gl=pyglet.gl.GL_POINTS)
        self.x = x
        self.y = y
        self.point_size = point_size # only for point

    def update_points(self):
        self.points = (self.x, self.y)



import math
import pyglet

from pyleap.shape.shape import Shape


class Polygon(Shape):
    """
    多边形：Polygon
    """
    def __init__(self, *args):
        """ 多边形
            颜色： color 默认为 "orange"
        """
        if isinstance(args[-1], str) or isinstance(args[-1], tuple) :
            color = args[-1]
            points = list(args[:len(args)-1])
        else:
            color = "orange"
            points = list(args)

        self._x = sum(points[::2]) / (len(points) // 2)
        self._y = sum(points[1::2]) / (len(points) // 2)
        self._move_x = 0
        self._move_y = 0

        super().__init__(color, gl=pyglet.gl.GL_POLYGON)
        self.points = self._points = points

    @property
    def x(self):
        return self._x + self._move_x
    
    @x.setter
    def x(self, x):
        self._move_x = x - self._x

    @property
    def y(self):
        return self._y + self._move_y
    
    @y.setter
    def y(self, y):
        self._move_y = y - self._y

    def update_points(self):
        if self._move_x==0 and self._move_y==0:
            self.points = self._points
            return

        ps = [self._move_x, self._move_y] * (len(self.points)//2)
        for i in range(len(ps)):
            ps[i] += self._points[i]
        
        self.points = tuple(ps)


class Triangle(Polygon):
    """ Triangle """

    def __init__(self, *args):
        super().__init__(*args)
        self.gl = pyglet.gl.GL_TRIANGLES
        


""" 基本图形： 线段

包含2个点

属性：
points： （x1, y1, x2, y2）
"""
from pyleap.shape.shape import Shape
import pyglet
import math


class Line(Shape):
    def __init__(self, x1=100, y1=100, x2=200, y2=200, line_width=1, color="orange"):
        """ 线段
        线段宽度： line_width， 默认为1
        颜色： color, 默认为"orange"
        """
        super().__init__(color, gl=pyglet.gl.GL_LINES, line_width=line_width)
        self.x1, self.y1, self.x2, self.y2 = x1, y1, x2, y2

    def update_points(self):
        self.points = (self.x1, self.y1, self.x2, self.y2)
    
    def draw(self):
        self.stroke()



from pyleap.constant_colors import color_dict


def color_to_tuple(color, opacity=1):
    """ convert any color to standard ()
    "red"       ->  'c3B', (255, 125, 0)
    "#ffffff"   ->  'c3B', (255, 255, 255)
    "#ffffffff" ->  'c4B', (255, 255, 255, 255)
    """
    if(type(color) == str and color[0] == "#"):
        color = hex_color_to_tuple(color)

    elif type(color) == str:
        if color in color_dict:
            color = color_dict[color.lower()]
        else:
            print("无法解析颜色: " + color)
            color = (255, 125, 0, int(255*opacity)) 

    while len(color) < 4:
        color += (int(255*opacity),)

    return color

def hex_color_to_tuple(hex):
    """ convent hex color to tuple
    "#ffffff"   ->  (255, 255, 255)
    "#ffff00ff" ->  (255, 255, 0, 255)
    """
    hex = hex[1:]
    length = len(hex) // 2
    return tuple(int(hex[i*2:i*2+2], 16) for i in range(length))

def hsl(h, s=0.5, l=0.5):
    return hsla_to_rgba(h, s, l, 1)

def hsla(h, s=0.5, l=0.5, a=1):
    return hsla_to_rgba(h, s, l, a)

def hsla_to_rgba(h, s, l, a):
    """ 0 <= H < 360, 0 <= s,l,a < 1
    """
    h = h % 360
    s = max(0, min(1, s))
    l = max(0, min(1, l))
    a = max(0, min(1, a))

    c = (1 - abs(2*l - 1)) * s
    x = c * (1 - abs(h/60%2 - 1))
    m = l - c/2

    if h<60:
        r, g, b = c, x, 0
    elif h<120:
        r, g, b = x, c, 0
    elif h<180:
        r, g, b = 0, c, x
    elif h<240:
        r, g, b = 0, x, c
    elif h<300:
        r, g, b = x, 0, c
    else:
        r, g, b = c, 0, x


    return (int((r+m)*255), int((g+m)*255), int((b+m)*255), int(a*255))


import pyglet
from pyleap.shape.shape import Shape


class Rectangle(Shape):
    """ 长方形： Rectangle
    """

    def __init__(self, x=100, y=100, w=100, h=50, color="orange"):
        """ 长方形
        左下角顶点的位置： x, y
        宽度： w
        高度： h
        颜色： color 默认为 "orange"
        """
        super().__init__(color, gl=pyglet.gl.GL_QUADS)
        self.x, self.y, self.w, self.h = x, y, w, h

    def update_points(self):
        """ 统一变为多个点组成的多边形，用于处理碰撞 """
        x, y, w, h = self.x, self.y, self.w, self.h
        self.points = (x, y, x + w, y, x + w, y + h, x, y + h)



import pyglet

from pyleap.resource import rss
from pyleap.window import window
from pyleap.shape.rectangle import Rectangle

cache_images = {}


class Sprite(Rectangle):
    """ Sprite """

    def __init__(self, src, x=300, y=200, w=None, h=None, batch=None):
        """
        默认位置： 300, 200
        """
        self.batch = batch
        self.src = src
        w = w or self._sprite.width
        h = h or self._sprite.height
        super().__init__(x, y, w, h)
        self.collision_scale = 0.8

    @property
    def w(self):
        return self.img.width * self._sprite.scale_x
    
    @w.setter
    def w(self, w):
        self._sprite.scale_x =  w / self.img.width

    @property
    def h(self):
        return self.img.height * self._sprite.scale_y
    
    @h.setter
    def h(self, h):
        self._sprite.scale_y =  h / self.img.height

    @property
    def src(self):
        return self._src
    
    @src.setter
    def src(self, src):
        if src not in cache_images:
            try:
                file_name = rss.get(src);
                if file_name[-4:] == ".gif":
                    img = pyglet.image.load_animation(file_name)
                    img.height = img.get_max_height()
                    img.width = img.get_max_width()
                else:
                    img = pyglet.image.load(file_name)
            except pyglet.image.codecs.ImageDecodeException:
                import os, sys
                print("无法加载图片,请检测图片是否可以正常显示，建议删除该图片后重试")
                print("错误图片：{}\\{}".format(os.getcwd(), rss.get(src)))
                sys.exit()

            
            self.center_image(img)
            cache_images[src] = img

        self._src = src
        self.img = cache_images[src]
        
        if hasattr(self, "_sprite"):
            self._sprite.image = self.img
        else:
            try:
                self._sprite = pyglet.sprite.Sprite(img=self.img, batch=self.batch)
            except pyglet.gl.lib.GLException:
                from ctypes import c_long
                import sys,os
                i = c_long()
                pyglet.gl.glGetIntegerv(pyglet.gl.GL_MAX_TEXTURE_SIZE, i)
                print("你的显卡支持的最大图片长宽为：{}x{}".format(i.value, i.value))
                print("请将图片大小调整为合适大小后重试")
                print("错误图片：{}\\{}".format(os.getcwd(), rss.get(src)))
                sys.exit()


    @property
    def x(self):
        return self._sprite.x

    @x.setter
    def x(self, x):
        self._sprite.x = x

    @property
    def y(self):
        return self._sprite.y

    @y.setter
    def y(self, y):
        self._sprite.y = y

    @property
    def opacity(self):
        return self._sprite.opacity / 255

    @opacity.setter
    def opacity(self, opacity):
        self._sprite.opacity = int(opacity * 255)

    def center_image(self, img):
        """Sets an image's anchor point to its center"""
        img.anchor_x = img.width // 2 # int
        img.anchor_y = img.height // 2 # int

    def update_points(self):
        min_x = self.x - self.w/2 * self.collision_scale
        min_y = self.y - self.h/2 * self.collision_scale
        max_x = self.x + self.w/2 * self.collision_scale
        max_y = self.y + self.h/2 * self.collision_scale
        self.points = (min_x, min_y, max_x, min_y, max_x, max_y, min_x, max_y)

    def draw(self):
        self._before_draw()
        if(type(self._sprite.image)==pyglet.image.Animation):
            pyglet.gl.glTranslatef(-self.w//2, -self.h//2, 0)
        self._sprite.draw()
        self._after_draw()

    def delete(self):
        self._sprite.delete()



import pyglet
import sys
import os
import configparser

__all__ = ['null', 'run', 'stop', 'repeat', 'run_after', 'all_shapes',
        'all_right_shapes', 'P', 'get_fps', 'Batch', 'config', 'screen']

get_fps = pyglet.clock.get_fps
Batch = pyglet.graphics.Batch
stop = pyglet.clock.unschedule

display = pyglet.canvas.get_display()
screen = display.get_default_screen()

def null():
    """ 空函数 """
    pass

def repeat(f, dt=1/60):
    """ 重复执行函数f，时间间隔dt """
    stop(f)
    pyglet.clock.schedule_interval_soft(f, dt)


def run_after(f, dt=1/60):
    """ 在一定时间之后执行 """
    pyglet.clock.schedule_once(f, dt)


def run():
    """ 在程序的最后运行，进入循环阻塞 """
    pyglet.app.run()
    

# all_shapes, use for track all the shapes draw on the windows
all_shapes = set()
all_right_shapes = set()


class P:
    """ P for point """
    def __init__(self, x, y):
        self.x = x
        self.y = y

# config file: download/config.ini

class Config:
    def __init__(self, src='download/config.ini'):
        self.src = src
        try:
            # user configs
            self.config = configparser.ConfigParser()
            self.config.read(self.src)
            if not os.path.exists("download"):
                os.makedirs("download")
        except:
            self.config = {}

    def save(self):
        try:
            with open(self.src, 'w') as configfile:
                self.config.write(configfile)
        except:
            pass

    def get_window_position(self):
        location_x = location_y = -1

        if 'location' in self.config:
            location_x = int(self.config['location']['x'])
            location_y = int(self.config['location']['y'])

        if location_x < 0 or location_y < 0 or screen.width < location_x or screen.height < location_y:
            location_x = screen.width // 2
            location_y = screen.height // 2
        return [location_x, location_y]

    def set_window_position(self, x, y):
        self.config['location'] = {'x': str(x),'y': str(y)}
        self.save()

    def set_window_quality(self, quality='normal'):
        if quality == self.get_window_quality(): return
        self.config['window'] = {'quality': quality}
        self.save()

    def get_window_quality(self):
        quality = 'normal'
        if 'window' in self.config:
            quality = self.config['window']['quality']

        return quality

config = Config()



from pyleap.shape.circle import Circle
from pyleap.shape.line import Line
from pyleap.shape.point import Point
from pyleap.shape.rectangle import Rectangle
from pyleap.shape.shape import Shape
from pyleap.shape.sprite import Sprite
from pyleap.shape.text import Text
from pyleap.shape.polygon import Polygon, Triangle
from pyleap.shape.ellipse import Ellipse


""" 音效对象
支持的音效格式有（不限于以下列出的格式）：

* AU
* MP2
* MP3
* OGG/Vorbis
* WAV
* WMA

在项目中使用音效，除wav格式的音效外，其他的音效资源需要安装Avbin才能播放。 Avbin下载地址：http://avbin.github.io/AVbin/Download.html
可以使用网络地址链接，如果使用的是网络地址链接，则会下载该资源到本地，下次运行时将直接读取本地资源
"""

import pyglet
from pyleap.resource import rss

__all__ = ['Audio', 'play', 'pause']

cache_players = {}

def Audio(url, loop=False):
    if url not in cache_players:
        source = pyglet.media.load(rss.get(url))
        player = pyglet.media.Player()
        player.queue(source)
        cache_players[url] = player
        player.loop = loop

    return cache_players[url]


def play(url, loop=False):
    Audio(url, loop).play()

def pause(url):
    """ 暂停播放音效 """
    Audio(url).pause()







"""基本图形：圆

圆是以圆心为中心的正多边形（正72边形）的形式进行拟合
"""

import pyglet
from math import sin, cos, pi, sqrt
from pyleap.shape.shape import Shape


class Circle(Shape):
    """ 基本图形：圆 Circle 
    """

    def __init__(self, x=100, y=100, r=30, color="orange"):
        """ 
        圆心： x、y,    默认为100, 100
        半径： r，     默认为30
        颜色： color,  默认为 "orange"
        """
        super().__init__(color, gl=pyglet.gl.GL_POLYGON)
        self.x, self.y, self.r = x, y, r

    def update_points(self):
        """ 圆的近似图形：正多边形 """
        n = 72
        d = pi * 2 / n
        x, y, r = self.x, self.y, self.r

        ps = []
        for i in range(n):
            ps += [(x + r * sin(d * i)), (y + r * cos(d * i))]
        self.points = tuple(ps)



""" 基本图形类 Shape

所有图形都继承与Shape，Shape拥有以下两个主要的属性
1. Collision，主要用于检测碰撞以及点击事件
2. Transform，用于图形的变换处理

"""

import pyglet
import math

from pyleap.transform import Transform, TransformMixin
from pyleap.collision import CollisionMixin
from pyleap.color import color_to_tuple
from pyleap.util import all_shapes, all_right_shapes, null


class Shape(CollisionMixin, TransformMixin):
    """ base shape class """


    def __init__(self, color="orange", gl=pyglet.gl.GL_LINE_LOOP,
                 line_width=1):
        """ 默认参数
        颜色 color: "orange"
        线条粗细 line_width： 1
        """
        self.color = color
        self.gl = gl
        self.transform = Transform()

        # 图形的线条宽度，默认为1
        self.line_width = line_width

        # 图形的顶点 (Point1, Point2, ...)
        self.points = ()

        # 用于记录press事件函数
        self._press = null
        self._right_press = null

        # 仅Point类point_size属性有效
        self.point_size = 1

        # 透明度 opacity
        self.opacity = 1

    def draw(self):
        """ 使用draw方法将图形绘制在窗口里 """
        self._before_draw()
        self.vertex_list.draw(self.gl)
        self._after_draw()
    
    def _after_draw(self):
        """ reset gl """
        pyglet.gl.glLoadIdentity()  # reset gl
        pyglet.gl.glLineWidth(1)

    def stroke(self):
        """ 使用stroke方法将图形绘制在窗口里，仅对基本的几何图形有效 """
        self._before_draw()
        length = len(self.points)
        
        # thick lines
        # if self.line_width == 1:
        #     if length==4:
        #         self.vertex_list.draw(pyglet.gl.GL_LINES)
        #     elif length > 4:
        #         self.vertex_list.draw(pyglet.gl.GL_LINE_LOOP)
        #     return

        # 
        color = color_to_tuple(self.color, self.opacity)
        if length == 4:
            x, y, x1, y1 = self.points[0:4]
            l = max(1, math.sqrt((x1-x)*(x1-x)+(y1-y)*(y1-y)))
            ly = (x1-x) / l * self.line_width / 2
            lx = - (y1-y) / l * self.line_width / 2

            points = [x-lx, y-ly, x+lx, y+ly, x1+lx, y1+ly, x1-lx, y1-ly]

            vertex_list = pyglet.graphics.vertex_list(
                4,
                ('v2f', points),
                ('c4B', color * 4))
            vertex_list.draw(pyglet.gl.GL_QUADS)

        elif length > 4:
            points = []
            for i in range(0, length, 2):
                x, y = self.points[i], self.points[i+1]
                x1, y1 = self.points[i-2], self.points[i-1]
                x2, y2 = self.points[(i+2) % length], self.points[(i+3) % length]

                l1 = max(1, math.sqrt((x1-x)*(x1-x)+(y1-y)*(y1-y)))
                l2 = max(1, math.sqrt((x2-x)*(x2-x)+(y2-y)*(y2-y)))

                nx1, ny1 = (x - x1) / l1, (y - y1) / l1
                nx2, ny2 = (x - x2) / l2, (y - y2) / l2
                nx, ny = nx1 + nx2, ny1 + ny2
                vx, vy = -ny1, nx1

                t = nx1*nx2 + ny1*ny2
                if t > 0.99:
                    lx = vx * self.line_width / 2
                    ly = vy * self.line_width / 2
                    points += [x-lx, y-ly, x+lx, y+ly, x+lx, y+ly, x-lx, y-ly]
                elif t < -0.99:
                    lx = vx * self.line_width / 2
                    ly = vy * self.line_width / 2
                    points += [x+lx, y+ly, x-lx, y-ly]
                else:
                    radio = 1/(vx*nx + vy*ny)
                    if radio < 0: 
                        radio = -radio

                    lx = (nx1+nx2) * self.line_width / 2 * radio
                    ly = (ny1+ny2) * self.line_width / 2 * radio
                    points += [x+lx, y+ly, x-lx, y-ly]

            batch = pyglet.graphics.Batch()
            for i in range(0, len(points), 4):
                batch.add(4, pyglet.gl.GL_QUADS, None,
                    ('v2f', (points[i-4], points[i-3], points[i-2], points[i-1],
                        points[i+2], points[i+3], points[i], points[i+1])),
                    ('c4B', color * 4))

            batch.draw()

        self._after_draw()

    def _before_draw(self):
        """ 在绘制之前，针对形变进行计算，通过设置openGL的属性来达到绘制出变形的图形 """
        self.update_points()
        self.update_vertex_list()
        self.update_anchor()

        pyglet.gl.glLoadIdentity()  # reset gl
        pyglet.gl.glLineWidth(self.line_width)
        pyglet.gl.glPointSize(self.point_size)
        self.transform.update_gl()

        # handle shapes click envets
        all_shapes.discard(self)
        if(self._press != None):
            all_shapes.add(self)

        all_right_shapes.discard(self)
        if(self._right_press != None):
            all_right_shapes.add(self)  

    def update_points(self):
        """ translate shapes to points，在子类中实现 """
        pass

    def update_vertex_list(self):
        """ 使用pyglet来绘制基本图形之前，转为pyglet识别的属性 """
        color = color_to_tuple(self.color, self.opacity)
        length = len(self.points) // 2
        self.vertex_list = pyglet.graphics.vertex_list(
            length,
            ('v2f', self.points),
            ('c4B', color * length))

    def update_anchor(self):
        """ 如果是使用set_anchor_rate来设定锚点，那么就需要不停的更新锚点的位置 """
        t = self.transform
        self.update_collision_rect()
        if t.anchor_x_r and t.anchor_y_r:
            t.anchor_x = self.min_x + (self.max_x - self.min_x) * t.anchor_x_r
            t.anchor_y = self.min_y + (self.max_y - self.min_y) * t.anchor_y_r

__all__ = ["Shape"]


import setuptools

# pip install twine wheel

# window: rmdir /s/q dist
# python setup.py sdist bdist_wheel
# twine upload dist/* -u VicWang -p PYPI******

setuptools.setup(
    name="pyleap",
    version="2021.05.31",
    author="Vic Wang",
    author_email="305880887@qq.com",
    description='A simple 2D game engine base on pyglet',
    long_description='A simple 2D game engine base on pyglet',
    long_description_content_type="text/markdown",
    url='http://github.com/XRDX/pyleap',
    packages=setuptools.find_packages(),
    install_requires=[
        'pyglet==1.5.11',
        'leaptask>=2020.12.14'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
)



import pyglet
from pyleap.window import window
from pyleap.color import color_to_tuple
from pyleap.util import all_shapes

from pyleap.shape.rectangle import Rectangle


class Text(Rectangle):
    """ Text
    src: 文本内容
    font_size: 文字高度
    """

    def __init__(self, src, x=window.center_x, y=window.center_y,
                font_size=16, color="orange", font_name=None):
        """ """
        self.text = pyglet.text.Label(text=src, font_size=font_size, font_name=font_name)
        super().__init__(x=x, y=y, h=font_size, color=color)
        self.w = self.text.content_width
        self.h = self.text.content_height

    def draw(self):
        self._before_draw()
        self.text.x = self.x
        self.text.y = self.y
        self.text.color = color_to_tuple(self.color)
        self.text.draw()
        self._after_draw()

    @property
    def src(self):
        return self.text.text

    @src.setter
    def src(self, src):
        self.text.text = src
        self.w = self.text.content_width

    @property
    def font_size(self):
        return self.text.font_size

    @font_size.setter
    def font_size(self, font_size):
        self.text.font_size = font_size
        self.w = self.text.content_width
        self.h = self.text.content_height 



import math
import pyglet
from pyleap.collision import CollisionMixin

#     # x, y, z
#     gl.glLoadIdentity()
#     gl.glTranslatef(x, y, z=0)
#     gl.glScalef(x, y, z=1.0)
#     gl.glRotatef(30, 0.0, 0.0, 1.0)
#     gl.glTranslatef(-100, -100, 0)

class Transform(CollisionMixin):


    def __init__(self):
        self.anchor_x = 0
        self.anchor_y = 0
        self.anchor_x_r = 0.5
        self.anchor_y_r = 0.5
        self.rotation = 0
        self.scale = 1
        self.scale_x = 1
        self.scale_y = 1
    
    def is_transformed(self):
        return self.rotation != 0 or self.scale*self.scale_x!=1 or self.scale*self.scale_y!=1

    def set_anchor(self, x, y):
        self.anchor_x = x
        self.anchor_y = y
        self.anchor_x_r = None
        self.anchor_y_r = None

    def set_anchor_rate(self, x, y):
        self.anchor_x_r = x
        self.anchor_y_r = y

    def get_point(self, x, y):
        x -= self.anchor_x
        y -= self.anchor_y

        x *= self.scale_x * self.scale
        y *= self.scale_y * self.scale

        d = self.rotation / 180 * math.pi
        cos = math.cos(d)
        sin = math.sin(d)

        new_x = x*cos - y*sin
        new_y = y*cos + x*sin

        x = new_x
        y = new_y

        x += self.anchor_x
        y += self.anchor_y

        return (x, y)

    def update_points(self, ps):
        if not self.is_transformed():
            self.points = ps
            return
    
        self.points = ()
        for i in range(0, len(ps), 2):
            self.points += self.get_point(ps[i], ps[i+1])

    def update_gl(self):
        if not self.is_transformed(): 
            return

        pyglet.gl.glTranslatef(self.anchor_x, self.anchor_y, 0)
        pyglet.gl.glRotatef(self.rotation, 0.0, 0.0, 1.0)
        pyglet.gl.glScalef(self.scale_x*self.scale, self.scale_y*self.scale, 1.0)
        pyglet.gl.glTranslatef(-self.anchor_x, -self.anchor_y, 0)


class TransformMixin():

    # Transform methods
    @property
    def rotation(self):
        return self.transform.rotation

    @rotation.setter
    def rotation(self, rotation):
        self.transform.rotation = rotation

    @property
    def scale(self):
        return self.transform.scale

    @scale.setter
    def scale(self, scale):
        self.transform.scale = scale

    @property
    def scale_x(self):
        return self.transform.scale_x

    @scale_x.setter
    def scale_x(self, scale_x):
        self.transform.scale_x = scale_x   
           
    @property
    def scale_y(self):
        return self.transform.scale_y

    @scale_y.setter
    def scale_y(self, scale_y):
        self.transform.scale_y = scale_y  

    def set_anchor(self, x, y):
        self.transform.set_anchor(x, y)

    def set_anchor_rate(self, x_r, y_r):
        self.transform.set_anchor_rate(x_r, y_r) 

    def flip(self):
        self.scale_x = -self.scale_x

    def v_flip(self):
        self.scale_y = -self.scale_y



import pyglet
import platform

from pyleap.mouse import mouse
from pyleap.collision import shape_clicked
from pyleap.util import all_shapes, all_right_shapes, config, get_fps
from pyleap.key import key
from pyleap.resource import rss

__all__ = ['window', "Window"]

# disable debug gl option 
pyglet.options['debug_gl'] = False


def enable_smooth_multisample_blend():
    # 抗锯齿
    pyglet.gl.glEnable(pyglet.gl.GL_LINE_SMOOTH)
    pyglet.gl.glHint(pyglet.gl.GL_LINE_SMOOTH_HINT, pyglet.gl.GL_DONT_CARE)

    pyglet.gl.glEnable(pyglet.gl.GL_POLYGON_SMOOTH)
    pyglet.gl.glHint(pyglet.gl.GL_POLYGON_SMOOTH_HINT, pyglet.gl.GL_DONT_CARE)

    # 抗锯齿-多样本缓冲(Multisample Buffer)
    pyglet.gl.glEnable(pyglet.gl.GL_MULTISAMPLE)

    # 支持透明
    pyglet.gl.glBlendFunc(pyglet.gl.GL_SRC_ALPHA, pyglet.gl.GL_ONE_MINUS_SRC_ALPHA)
    pyglet.gl.glEnable(pyglet.gl.GL_BLEND)

class Window(pyglet.window.Window):
    """ 
    属性 Attributes

    caption
    str，窗口标题，只读.

    width/w
    窗口宽度，单位像素点，只读。

    height/h
    窗口高度，单位像素点，只读。

    方法 Methods

    clear()
    清除窗口里的内容

    set_caption(caption)
    设置窗口标题
    Parameters: caption (str or unicode)

    set_size(width, height)
    设置窗口大小
    """

    def __init__(self):
        """ 初始化，创建一个窗口 """
        # low quality window
        if config.get_window_quality() == 'low':
            super().__init__()
        # Mac or Linux
        elif platform.system() != "Windows": 
            super().__init__()
            enable_smooth_multisample_blend()
        
        elif platform.system() == "Windows":
            template = pyglet.gl.Config(alpha_size=8, sample_buffers=1, samples=4)
            configs = Window.get_matching_configs(template)
            if configs:
                super().__init__(config=configs[0])
                enable_smooth_multisample_blend()
            else:
                super().__init__()
    
        self.set_caption("LeapLearner")

        location_x, location_y = config.get_window_position()
        self.set_location(location_x, location_y)
        self.axis_batch = None
        self.set_size()
        init_event(self)

    @property
    def center_x(self):
        return self.width // 2

    @property
    def center_y(self):
        return self.height // 2

    @property
    def caption(self):
        return self._title
    
    def set_caption(self, caption):
        self._title = caption

    def set_size(self, w=640, h=480):
    	super().set_size(w, h)
    	self.w = w
    	self.h = h

    def update_caption(self, mouse):
        """ 添加坐标显示 """
        caption = "{}  x: {}, y: {}".format(self._title, mouse.x, mouse.y)
        super().set_caption(caption)

    def clear(self):
        all_shapes.clear()
        all_right_shapes.clear()
        # super().clear()
        pyglet.graphics.vertex_list(4,
            ('v2i', (0, 0, self.w, 0, self.w, self.h, 0, self.h)),
            ('c3B', (255, 255, 255) * 4)
        ).draw(pyglet.gl.GL_QUADS)

    def show_axis(self):       
        if self.axis_batch is None:
            self.create_axis_batch()

        self.axis_batch.draw()

    def create_axis_batch(self):
        self.axis_batch = pyglet.graphics.Batch()
        for x in range(0, self.w, 100):
            self.axis_batch.add(
                2, pyglet.gl.GL_LINES, None,
                ('v2i', (x, 0, x, self.h)),
                ('c3B', (240, 160, 0) * 2))

            pyglet.text.Label(str(x), x=x+2, y=2, color=(240, 160, 0, 255), batch=self.axis_batch)

        for y in range(0, self.h, 100):
            self.axis_batch.add(
                2, pyglet.gl.GL_LINES, None,
                ('v2i', (0, y, self.w, y)),
                ('c3B', (240, 160, 0) * 2))
            pyglet.text.Label(str(y), x=2, y=y+2, color=(240, 160, 0, 255), batch=self.axis_batch)

    def show_fps(self):
        fps = int(get_fps())

        if fps>100:
            s = "fps: >100"
        else:
            s = "fps: {}".format(fps)
        pyglet.text.Label(s, x=2, y=window.h-15, color=(240, 160, 0, 255)).draw()

    def keep_on_top(self):
        # keep window on top
        if platform.system()=="Windows":
            try:
                import win32gui
                import win32con
                win32gui.SetWindowPos(self._hwnd, win32con.HWND_TOPMOST, 0,0,0,0,
                win32con.SWP_NOMOVE | win32con.SWP_NOSIZE)
            except:
                print("Please install pywin32 first to use keep_on_top()")
        # Mac
        elif "Darwin" in platform.system():
            import os
            script = 'tell application "System Events" \
              to set frontmost of the first process whose unix id is {pid} to true'.format(pid=os.getpid())
            os.system("/usr/bin/osascript -e '{script}'".format(script=script))
        else:
            print("window.keep_on_top() is only supported on windows")

    def set_quality(self, quality='low'):
        config.set_window_quality(quality)


def init_event(window):
    @window.event
    def on_mouse_motion(x, y, dx, dy):
        """ 当鼠标没有按下时移动的时候触发 """
        mouse.x, mouse.y = x, y
        mouse.move()
        window.update_caption(mouse)


    @window.event
    def on_mouse_drag(x, y, dx, dy, buttons, modifiers):
        """ 当鼠标按下并且移动的时候触发 """
        mouse.x, mouse.y = x, y
        mouse.move()

    @window.event
    def on_mouse_press(x, y, button, modifiers):
        """ 按下鼠标时 

        """ 
        if button == 1: #MouseKeyCode.LEFT:
            mouse.press()
                    # 判断是否有图形的点击事件被触发了
            shapes = list(all_shapes)
            while shapes:
                shape = shapes.pop()
                if(shape._press and shape_clicked(shape)):
                    shape._press()
        elif button == 4: #MouseKeyCode.RIGHT:
            mouse.right_press()

            # 判断是否有图形的点击事件被触发了
            shapes = list(all_right_shapes)
            while shapes:
                shape = shapes.pop()
                if(shape._right_press and shape_clicked(shape)):
                    shape._right_press()



    @window.event
    def on_mouse_release(x, y, button, modifiers):
        """ 松开鼠标时 """ 
        if button == 1: #MouseKeyCode.LEFT:
            mouse.release()
        elif button == 4: #MouseKeyCode.RIGHT:
            mouse.right_release()


    @window.event
    def on_key_press(symbol, modifiers):
        """ 当键盘按键按下时触发 """
        try:
            key[symbol].press()
        except:
            pass


    @window.event
    def on_key_release(symbol, modifiers):
        """ 当键盘按键松开时触发 """
        try:
            key[symbol].release()
        except:
            pass

    @window.event
    def on_move(x, y):
        config.set_window_position(x, y)

try:
    window = Window()
except Exception as e:
    config.set_window_quality("low")
    window = Window()

icon1 = pyglet.image.load(rss.get('https://rss.leaplearner.com/assets/pc/favicon.ico'))
icon2 = pyglet.image.load(rss.get('https://rss.leaplearner.com/assets/pc/favicon.ico'))
window.set_icon(icon1, icon2)